<?php

namespace Drupal\image_sizes\Plugin\ImageEffect;

use Drupal\Core\Image\ImageInterface;
use Drupal\image\Plugin\ImageEffect\ResizeImageEffect;

/**
 * Crops an image resource.
 *
 * @ImageEffect(
 *   id = "image_sizes_ratio",
 *   label = @Translation("Scale to ratio"),
 *   description = @Translation("Resizing will make images an exact set of dimensions. This may cause images to be stretched or shrunk disproportionately.")
 * )
 */
class RatioEffect extends ResizeImageEffect {

  /**
   * {@inheritdoc}
   */
  public function applyEffect(ImageInterface $image) {
    $ratio = $this->configuration['width'] / $this->configuration['height'];

    $dimensions['height'] = min($image->getHeight(), $image->getWidth() / $ratio);
    $dimensions['width'] = min($image->getWidth(), $image->getHeight() * $ratio);

    if (!$image->scaleAndCrop($dimensions['width'], $dimensions['height'])) {
      $this->logger->error(
        'Image scale and crop failed using the %toolkit toolkit on %path (%mimetype, %dimensions)',
        [
          '%toolkit' => $image->getToolkitId(),
          '%path' => $image->getSource(),
          '%mimetype' => $image->getMimeType(),
          '%dimensions' => $image->getWidth() . 'x' . $image->getHeight(),
        ]
      );
      return FALSE;
    }
    return TRUE;
  }

}
